@@ -1,31 +1,55 @@
 
 #' Make a natural_join node.
 #'
-#' Natural join is a join by identity on all common columns
-#' (or only common columns specified in a non-\code{NULL} \code{by} argument).
-#' Any common columns not specified in a non-\code{NULL} \code{by} argument
+#' Natural join is a join by identity on all common columns specified in the \code{by}
+#' argument.
+#' Any common columns not specified in the \code{by} argument
 #' are coalesced into a single column prefering the first or "a" table.
 #'
 #' @param a source to select from.
 #' @param b source to select from.
 #' @param ... force later arguments to bind by name
+#' @param by character, set of columns to match.
 #' @param jointype type of join ('INNER', 'LEFT', 'RIGHT', 'FULL').
-#' @param by set of columns to match.
 #' @return natural_join node.
 #'
 #' @examples
 #'
-#' if (requireNamespace("RSQLite", quietly = TRUE)) {
-#'   my_db <- DBI::dbConnect(RSQLite::SQLite(), ":memory:")
-#'   d1 <- dbi_copy_to(my_db, 'd1',
-#'                     data.frame(AUC = 0.6, R2 = 0.2, D = NA))
-#'   d2 <- dbi_copy_to(my_db, 'd2',
-#'                     data.frame(AUC = 0.6, D = 0.3))
-#'   optree <- natural_join(d1, d2, by = 'AUC')
-#'   cat(format(optree))
-#'   sql <- to_sql(optree, my_db)
-#'   cat(sql)
-#'   print(DBI::dbGetQuery(my_db, sql))
+#' if(requireNamespace("DBI", quietly = TRUE) &&
+#'    requireNamespace("RSQLite", quietly = TRUE)) {
+#'   my_db <- DBI::dbConnect(RSQLite::SQLite(),
+#'                           ":memory:")
+#'
+#'   d1 <- rq_copy_to(
+#'     my_db, 'd1',
+#'     build_frame(
+#'       "key", "val", "val1" |
+#'         "a"  , 1  ,  10    |
+#'         "b"  , 2  ,  11    |
+#'         "c"  , 3  ,  12    ))
+#'   d2 <- rq_copy_to(
+#'     my_db, 'd2',
+#'     build_frame(
+#'       "key", "val", "val2" |
+#'         "a"  , 5  ,  13    |
+#'         "b"  , 6  ,  14    |
+#'         "d"  , 7  ,  15    ))
+#'
+#'   # key matching join
+#'   optree <- natural_join(d1, d2,
+#'                          jointype = "LEFT", by = 'key')
+#'   execute(my_db, optree) %.>%
+#'     print(.)
+#'
+#'   # full cross-product join
+#'   # (usually with jointype = "FULL", but "LEFT" is more
+#'   # compatible with rquery field merg semantics).
+#'   optree2 <- natural_join(d1, d2,
+#'                           jointype = "LEFT", by = NULL)
+#'   execute(my_db, optree2) %.>%
+#'     print(.)
+#'   # notice ALL non-"by" fields take coalese to left table.
+#'
 #'   DBI::dbDisconnect(my_db)
 #' }
 #'
@@ -34,15 +58,15 @@
 natural_join <- function(a, b,
                          ...,
                          jointype = 'INNER',
-                         by = NULL) {
+                         by) {
   UseMethod("natural_join", a)
 }
 
 #' @export
 natural_join.relop <- function(a, b,
                                ...,
-                               jointype = 'INNER',
-                               by = NULL) {
+                               by,
+                               jointype = 'INNER') {
   if(length(list(...))>0) {
     stop("rquery::natural_join unexpected arguments")
   }
@@ -52,14 +76,10 @@ natural_join.relop <- function(a, b,
   usesa <- column_names(a)
   usesb <- column_names(b)
   common <- intersect(usesa, usesb)
-  if(is.null(by)) {
-    by <- common
-    print(paste("rquery::natural_join.relop joining by ",
-                paste(by, collapse = ", ")))
-  } else {
+  if(!is.null(by)) {
     bads <- setdiff(by, common)
     if(length(bads)>0) {
-      stop(paste("rquery::natural_join.relop can not join by",
+      stop(paste("rquery::natural_join.relop all tables must have all join keys, the following keys are not in some tables:",
                  paste(bads, collapse = ", ")))
     }
   }
@@ -75,8 +95,8 @@ natural_join.relop <- function(a, b,
 #' @export
 natural_join.data.frame <- function(a, b,
                                     ...,
-                                    jointype = 'INNER',
-                                    by = NULL) {
+                                    by,
+                                    jointype = 'INNER') {
   if(length(list(...))>0) {
     stop("rquery::natural_join unexpected arguments")
   }
@@ -85,11 +105,9 @@ natural_join.data.frame <- function(a, b,
   }
   nmgen <- mk_tmp_name_source("rquery_tmp")
   tmp_namea <- nmgen()
-  dnodea <- table_source(tmp_namea, colnames(a))
-  dnodea$data <- a
+  dnodea <- mk_td(tmp_namea, colnames(a))
   tmp_nameb <- nmgen()
-  dnodeb <- table_source(tmp_namea, colnames(b))
-  dnodeb$data <- b
+  dnodeb <- mk_td(tmp_namea, colnames(b))
   enode <- natural_join(dnodea, dnodeb,
                         jointype = jointype,
                         by = by)
@@ -131,8 +149,7 @@ format.relop_natural_join <- function(x, ...) {
 
 
 calc_used_relop_natural_join <- function (x, ...,
-                                          using = NULL,
-                                          contract = FALSE) {
+                                          using = NULL) {
   cols <- column_names(x)
   if(length(using)>0) {
     missing <- setdiff(using, cols)
@@ -149,19 +166,15 @@ calc_used_relop_natural_join <- function (x, ...,
 
 #' @export
 columns_used.relop_natural_join <- function (x, ...,
-                                             using = NULL,
-                                             contract = FALSE) {
+                                             using = NULL) {
   using <- calc_used_relop_natural_join(x,
-                                        using=using,
-                                        contract = contract)
+                                        using=using)
   c1 <- intersect(using, column_names(x$source[[1]]))
   s1 <- columns_used(x$source[[1]],
-                     using = c1,
-                     contract = contract)
+                     using = c1)
   c2 <- intersect(using, column_names(x$source[[2]]))
   s2 <- columns_used(x$source[[2]],
-                     using = c2,
-                     contract = contract)
+                     using = c2)
   merge_columns_used(s1, s2)
 }
 
@@ -179,9 +192,16 @@ to_sql.relop_natural_join <- function (x,
   if(length(list(...))>0) {
     stop("unexpected arguments")
   }
-  using <- calc_used_relop_natural_join(x,
-                                        using=using)
-  c1 <- intersect(using, column_names(x$source[[1]]))
+  using <- unique(c(calc_used_relop_natural_join(x,
+                                        using=using),
+                    x$by))
+  cs1 <- column_names(x$source[[1]])
+  cs2 <- column_names(x$source[[2]])
+  if(length(setdiff(using, c(cs1, cs2)))>0) {
+    stop(paste("to_sql.relop_natural_join input table(s) missing columns ",
+               paste(setdiff(using, c(cs1, cs2)), collapse = ", ")))
+  }
+  c1 <- intersect(using, cs1)
   subsqla_list <- to_sql(x$source[[1]],
                          db = db,
                          source_limit = source_limit,
@@ -190,7 +210,7 @@ to_sql.relop_natural_join <- function (x,
                          append_cr = FALSE,
                          using = c1)
   subsqla <- subsqla_list[[length(subsqla_list)]]
-  c2 <- intersect(using, column_names(x$source[[2]]))
+  c2 <- intersect(using, cs2)
   subsqlb_list <- to_sql(x$source[[2]],
                          db = db,
                          source_limit = source_limit,
@@ -204,8 +224,8 @@ to_sql.relop_natural_join <- function (x,
   tabb <- tnum()
   tabbq <- quote_identifier(db, tabb)
   bexpr <- NULL
-  aterms <- setdiff(column_names(x$source[[1]]), x$by)
-  bterms <- setdiff(column_names(x$source[[2]]), x$by)
+  aterms <- setdiff(c1, x$by)
+  bterms <- setdiff(c2, x$by)
   overlap <- c(x$by, intersect(aterms, bterms))
   prefix <- paste(rep(' ', indent_level), collapse = '')
   osql <- vapply(overlap,
