@@ -5,32 +5,36 @@
 #'
 #' Note: non-SQL nodes are allowed to delete/overwrite both
 #' both the incoming and outgoing tables, so do not point them
-#' to non-temporary structures.
+#' to non-temporary structures.  Also they tend to land all columns
+#' (losing narrowing optimization),
+#' so can be expensive and should be used sparingly.
 #'
 #' @param source source to work from (data.frame or relop node)
-#' @param f implementation signature: f(db, incoming_table_name, outgoing_table_name)
 #' @param ... force later arguments to bind by name
+#' @param f_db database implementation signature: f_db(db, incoming_table_name, outgoing_table_name) (db being a database handle)
+#' @param f_df data.frame implementation signature: f_df(data.frame) (NULL defaults to taking from database).
 #' @param incoming_table_name character, name of incoming table
-#' @param columns_used character, incoming columns used
 #' @param outgoing_table_name character, name of produced table
-#' @param columns_produced character, names of columns produced
+#' @param columns_produced character, names of additional columns produced
 #' @param display_form chacter, how to print node
+#' @param pass_using logical, if TRUE (or if f_db is NULL) pass using column calculations through (else assume using all columns).
 #' @param orig_columns logical if TRUE select all original columns.
 #' @param temporary logical, if TRUE mark tables temporary.
 #' @return sql node.
 #'
-#' @seealso \code{\link{rsummary_node}}
+#' @seealso \code{\link{rsummary_node}}, \code{\link{quantile_node}}, \code{\link{materialize_node}}
 #'
 #' @export
 #'
 non_sql_node <- function(source,
-                         f,
                          ...,
+                         f_db,
+                         f_df = NULL,
                          incoming_table_name,
-                         columns_used,
                          outgoing_table_name,
                          columns_produced,
                          display_form,
+                         pass_using = FALSE,
                          orig_columns = TRUE,
                          temporary = TRUE) {
   wrapr::stop_if_dot_args(substitute(list(...)), "non_sql_node")
@@ -39,30 +43,32 @@ non_sql_node <- function(source,
 
 #' @export
 non_sql_node.relop <- function(source,
-                               f,
                                ...,
+                               f_db,
+                               f_df = NULL,
                                incoming_table_name,
-                               columns_used,
                                outgoing_table_name,
                                columns_produced,
                                display_form,
+                               pass_using = FALSE,
                                orig_columns = TRUE,
                                temporary = TRUE) {
   wrapr::stop_if_dot_args(substitute(list(...)), "non_sql_node.relop")
-  if(is.null(f)) {
+  if(is.null(f_db)) {
     if(incoming_table_name!=outgoing_table_name) {
-      stop("non_sql_node.relop: must have incoming_table_name==outgoing_table_name when f is NULL")
+      stop("non_sql_node.relop: must have incoming_table_name==outgoing_table_name when f_db is NULL")
     }
   } else {
     if(incoming_table_name==outgoing_table_name) {
-      stop("non_sql_node.relop: must have incoming_table_name!=outgoing_table_name when f is not NULL")
+      stop("non_sql_node.relop: must have incoming_table_name!=outgoing_table_name when f_db is not NULL")
     }
   }
   r <- list(source = list(source),
             table_name = outgoing_table_name,
-            f = f,
+            f_db = f_db,
+            f_df = f_df,
+            pass_using = pass_using || is.null(f_db),
             incoming_table_name = incoming_table_name,
-            columns_used = columns_used,
             outgoing_table_name = outgoing_table_name,
             columns_produced = columns_produced,
             display_form = display_form,
@@ -75,26 +81,27 @@ non_sql_node.relop <- function(source,
 
 #' @export
 non_sql_node.data.frame <- function(source,
-                                    f,
                                     ...,
+                                    f_db,
+                                    f_df = NULL,
                                     incoming_table_name,
-                                    columns_used,
                                     outgoing_table_name,
                                     columns_produced,
                                     display_form,
+                                    pass_using = FALSE,
                                     orig_columns = TRUE,
                                     temporary = TRUE) {
   wrapr::stop_if_dot_args(substitute(list(...)), "non_sql_node.data.frame")
   tmp_name <- mk_tmp_name_source("rquery_tmp")()
-  dnode <- table_source(tmp_name, colnames(source))
-  dnode$data <- source
+  dnode <- mk_td(tmp_name, colnames(source))
   enode <- non_sql_node(source,
-                        f,
+                        f_db = f_db,
+                        f_df = f_df,
                         incoming_table_name = incoming_table_name,
-                        columns_used = columns_used,
                         outgoing_table_name = outgoing_table_name,
                         columns_produced = columns_produced,
                         display_form = display_form,
+                        pass_using = pass_using,
                         orig_columns = orig_columns,
                         temporary = temporary)
   return(enode)
@@ -124,13 +131,13 @@ format_node.relop_non_sql <- function(node) {
 
 #' @export
 columns_used.relop_non_sql <- function (x, ...,
-                                        using = NULL,
-                                        contract = FALSE) {
-  # assume using all columns
-  # TODO: improve
+                                        using = NULL) {
+  usingQ <- NULL
+  if(x$pass_using) {
+    usingQ <- using
+  }
   return(columns_used(x$source[[1]],
-                      using = NULL,
-                      contract = contract))
+                      using = usingQ))
 }
 
 
@@ -144,13 +151,17 @@ to_sql.relop_non_sql <- function (x,
                                   tnum = mk_tmp_name_source('tsql'),
                                   append_cr = TRUE,
                                   using = NULL) {
+  usingQ <- NULL
+  if(x$pass_using) {
+    usingQ <- using
+  }
   subsql <- to_sql(x$source[[1]],
                     db = db,
                     source_limit = source_limit,
                     indent_level = indent_level + 1,
                     tnum = tnum,
                     append_cr = append_cr,
-                    using = NULL)
+                    using = usingQ)
   nsubsql <- length(subsql)
   # non-SQL nodes must always be surrounded by SQL on both sides
   step1 <- materialize_sql_statement(db,
@@ -161,21 +172,21 @@ to_sql.relop_non_sql <- function (x,
                     incoming_table_name = x$incoming_table_name,
                     outgoing_table_name = x$outgoing_table_name,
                     temporary = x$temporary,
-                    f = x$f)
+                    f = x$f_db)
   class(nsql_step) <- "rquery_non_sql_step"
   step2 <- list(nsql_step)
   qlimit = limit
   if(!getDBOption(db, "use_pass_limit", TRUE)) {
     qlimit = NULL
   }
-  step3 <- list(to_sql(table_source(x$outgoing_table_name, column_names(x)),
+  step3 <- list(to_sql(mk_td(x$outgoing_table_name, column_names(x)),
                        db = db,
                        limit = qlimit,
                        source_limit = source_limit,
                        indent_level = indent_level + 1,
                        tnum = tnum,
                        append_cr = append_cr,
-                       using = NULL))
+                       using = usingQ))
   c(subsql[-length(subsql)], step1, step2, step3)
 }
 
@@ -191,3 +202,33 @@ print.rquery_non_sql_step <- function(x, ...) {
   print(format(x))
 }
 
+
+#' Cache results to a named table inside a pipeline.
+#'
+#'
+#' @param source source to work from (data.frame or relop node)
+#' @param table_name character, name of caching table
+#' @param ... force later arguments to bind by name
+#' @param temporary logical, if TRUE mark tables temporary.
+#' @return sql node.
+#'
+#' @seealso \code{\link{rsummary_node}}, \code{\link{non_sql_node}}
+#'
+#' @export
+#'
+materialize_node <- function(source,
+                             table_name,
+                             ...,
+                             temporary = TRUE) {
+  wrapr::stop_if_dot_args(substitute(list(...)), "materialize_node")
+  non_sql_node(source = source,
+               f_db = NULL,
+               f_df = function(x) { x },
+               incoming_table_name = table_name,
+               outgoing_table_name = table_name,
+               columns_produced = NULL,
+               display_form = paste0("materialize_node(", table_name, ")"),
+               orig_columns = TRUE,
+               temporary = temporary)
+}
+
